@@ -3,6 +3,7 @@ import torch
 from torch import nn
 from torch.nn.functional import one_hot
 from sklearn import metrics
+from ..datasets.lpkt_utils import generate_time2idx
 
 import pandas as pd
 
@@ -50,13 +51,20 @@ def evaluate(model, test_loader, model_name, save_path=""):
         y_scores = []
         dres = dict()
         for data in test_loader:
+            # if model_name in ["dkt_forget", "lpkt"]:
+            #     q, c, r, qshft, cshft, rshft, m, sm, d, dshft = data
             if model_name in ["dkt_forget"]:
-                q, c, r, qshft, cshft, rshft, m, sm, d, dshft = data
+                dcur, dgaps = data
             else:
-                q, c, r, qshft, cshft, rshft, m, sm = data
+                dcur = data
+            q, c, r = dcur["qseqs"], dcur["cseqs"], dcur["rseqs"]
+            qshft, cshft, rshft = dcur["shft_qseqs"], dcur["shft_cseqs"], dcur["shft_rseqs"]
+            m, sm = dcur["masks"], dcur["smasks"]
             q, c, r, qshft, cshft, rshft, m, sm = q.to(device), c.to(device), r.to(device), qshft.to(device), cshft.to(device), rshft.to(device), m.to(device), sm.to(device)
-
-            model.eval()
+            if model.model_name=='iekt':
+                model.model.eval()
+            else:
+                model.eval()
 
             # print(f"before y: {y.shape}")
             cq = torch.cat((q[:,0:1], qshft), dim=1)
@@ -66,9 +74,9 @@ def evaluate(model, test_loader, model_name, save_path=""):
                 y = model(c.long(), r.long())
                 y = (y * one_hot(cshft.long(), model.num_c)).sum(-1)
             elif model_name in ["dkt_forget"]:
-                y = model(c.long(), r.long(), d, dshft)
+                y = model(c.long(), r.long(), dgaps)
                 y = (y * one_hot(cshft.long(), model.num_c)).sum(-1)
-            elif model_name in ["dkvmn"]:
+            elif model_name in ["dkvmn", "skvmn"]:
                 y = model(cc.long(), cr.long())
                 y = y[:,1:]
             elif model_name in ["kqn", "sakt"]:
@@ -76,7 +84,7 @@ def evaluate(model, test_loader, model_name, save_path=""):
             elif model_name == "saint":
                 y = model(cq.long(), cc.long(), r.long())
                 y = y[:, 1:]
-            elif model_name == "akt":                                
+            elif model_name in ["akt", "akt_vector", "akt_norasch", "akt_mono", "akt_attn", "aktattn_pos", "aktmono_pos", "akt_raschx", "akt_raschy", "aktvec_raschx"]:                                
                 y, reg_loss = model(cc.long(), cr.long(), cq.long())
                 y = y[:,1:]
             elif model_name in ["atkt", "atktfix"]:
@@ -84,7 +92,19 @@ def evaluate(model, test_loader, model_name, save_path=""):
                 y = (y * one_hot(cshft.long(), model.num_c)).sum(-1)
             elif model_name == "gkt":
                 y = model(cc.long(), cr.long())
-                
+            elif model_name == "lpkt":
+                # cat = torch.cat((d["at_seqs"][:,0:1], dshft["at_seqs"]), dim=1).to(device)
+                cit = torch.cat((dcur["itseqs"][:,0:1], dcur["shft_itseqs"]), dim=1)
+                y = model(cq.long(), cr.long(), cit.long())
+                y = y[:,1:]  
+            elif model_name == "hawkes":
+                ct = torch.cat((dcur["tseqs"][:,0:1], dcur["shft_tseqs"]), dim=1)
+                # csm = torch.cat((dcur["smasks"][:,0:1], dcur["smasks"]), dim=1)
+                y = model(cc.long(), cq.long(), ct.long(), cr.long())#, csm.long())
+                y = y[:, 1:]
+            elif model_name == "iekt":
+                y = model.predict_one_step(data)
+                c,cshft = q,qshft#question level 
             # print(f"after y: {y.shape}")
             # save predict result
             if save_path != "":
@@ -108,11 +128,11 @@ def evaluate(model, test_loader, model_name, save_path=""):
     return auc, acc
 
 def early_fusion(curhs, model, model_name):
-    if model_name == "dkvmn":
+    if model_name in ["dkvmn", "skvmn"]:
         p = model.p_layer(model.dropout_layer(curhs[0]))
         p = torch.sigmoid(p)
         p = p.squeeze(-1)
-    elif model_name == "akt":
+    elif model_name in ["akt", "akt_vector", "akt_norasch", "akt_mono", "akt_attn", "aktattn_pos", "aktmono_pos", "akt_raschx", "akt_raschy", "aktvec_raschx"]:
         output = model.out(curhs[0]).squeeze(-1)
         m = nn.Sigmoid()
         p = m(output)
@@ -124,23 +144,33 @@ def early_fusion(curhs, model, model_name):
     elif model_name == "kqn":
         logits = torch.sum(curhs[0] * curhs[1], dim=1) # (batch_size, max_seq_len)
         p = model.sigmoid(logits)
+    elif model_name == "hawkes":
+        p = curhs[0].sigmoid()
+    elif model_name == "lpkt":
+        y = model.sig(model.linear_5(torch.cat((curhs[1], curhs[0]), 1))).sum(1) / self.d_k
+        p = curhs[0].sigmoid()
     return p
 
 def late_fusion(dcur, curdf, fusion_type=["mean", "vote", "all"]):
+    high, low = [], []
+    for pred in curdf["preds"]:
+        if pred >= 0.5:
+            high.append(pred)
+        else:
+            low.append(pred)
+
     if "mean" in fusion_type:
         dcur.setdefault("late_mean", [])
         dcur["late_mean"].append(round(curdf["preds"].mean().astype(float), 4))
     if "vote" in fusion_type:
         dcur.setdefault("late_vote", [])
-        labeldf = curdf["preds"] >= 0.5
-        counts = labeldf.value_counts()
-        poslen = 0 if True not in counts else counts[True]
-        dcur["late_vote"].append(1.0 if poslen / curdf.shape[0] >= 0.5 else 0.0)
+        correctnum = list(curdf["preds"]>=0.5).count(True)
+        late_vote = np.mean(high) if correctnum / len(curdf["preds"]) >= 0.5 else np.mean(low)
+        dcur["late_vote"].append(late_vote)
     if "all" in fusion_type:
         dcur.setdefault("late_all", [])
-        labeldf = curdf["preds"] >= 0.5
-        counts = labeldf.value_counts()
-        dcur["late_all"].append(1.0 if True in counts and False not in counts else 0.0)
+        late_all = np.mean(high) if correctnum == len(curdf["preds"]) else np.mean(low)
+        dcur["late_all"].append(late_all)
     return 
 
 def effective_fusion(df, model, model_name, fusion_type):
@@ -149,14 +179,18 @@ def effective_fusion(df, model, model_name, fusion_type):
 
     curhs, curr = [[], []], []
     dcur = {"late_trues": [], "qidxs": [], "questions": [], "concepts": [], "row": [], "concept_preds": []}
+    hasearly = ["dkvmn", "skvmn", "akt", "saint", "sakt", "hawkes", "akt_vector", "akt_norasch", "akt_mono", "akt_attn", "aktattn_pos", "aktmono_pos", "akt_raschx", "akt_raschy", "aktvec_raschx"]
     for ui in df:
         # 一题一题处理
         curdf = ui[1]
-        if model_name in ["dkvmn", "akt", "saint", "sakt"]:
+        if model_name in hasearly:
             curhs[0].append(curdf["hidden"].mean().astype(float))
         elif model_name == "kqn":
             curhs[0].append(curdf["ek"].mean().astype(float))
             curhs[1].append(curdf["es"].mean().astype(float))
+        elif model_name == "lpkt":
+            curhs[0].append(curdf["h"].mean().astype(float))
+            curhs[1].append(curdf["e_data"].mean().astype(float))
         else:
             # print(f"model: {model_name} has no early fusion res!")
             pass
@@ -175,7 +209,7 @@ def effective_fusion(df, model, model_name, fusion_type):
         dres.setdefault(key, [])
         dres[key].append(np.array(dcur[key]))
     # early fusion
-    if "early_fusion" in fusion_type and model_name in ["dkvmn", "akt", "saint", "sakt", "kqn"]:
+    if "early_fusion" in fusion_type and model_name in hasearly:
         curhs = [torch.tensor(curh).float().to(device) for curh in curhs]
         curr = torch.tensor(curr).long().to(device)
         p = early_fusion(curhs, model, model_name)
@@ -189,6 +223,8 @@ def group_fusion(dmerge, model, model_name, fusion_type, fout):
     hs, sms, cq, cc, rs, ps, qidxs, rests, orirows = dmerge["hs"], dmerge["sm"], dmerge["cq"], dmerge["cc"], dmerge["cr"], dmerge["y"], dmerge["qidxs"], dmerge["rests"], dmerge["orirow"]
     if cq.shape[1] == 0:
         cq = cc
+
+    hasearly = ["dkvmn", "skvmn", "kqn", "akt", "saint", "sakt", "hawkes", "akt_vector", "akt_norasch", "akt_mono", "akt_attn", "aktattn_pos", "aktmono_pos", "akt_raschx", "akt_raschy", "aktvec_raschx", "lpkt"]
     
     alldfs, drest = [], dict() # not predict infos!
     # print(f"real bz in group fusion: {rs.shape[0]}")
@@ -202,11 +238,14 @@ def group_fusion(dmerge, model, model_name, fusion_type, fout):
         # print(f"qid: {len(curqidxs)}, select: {len(cursm)}, response: {len(rs[bz].cpu().tolist())}, preds: {len(curps)}")
         df = pd.DataFrame({"qidx": curqidxs, "rest": currests, "row": currows, "select": cursm, 
                 "questions": cq[bz].cpu().tolist(), "concepts": cc[bz].cpu().tolist(), "response": rs[bz].cpu().tolist(), "preds": curps})
-        if model_name in ["dkvmn", "akt", "saint", "sakt"]:
+        if model_name in hasearly and model_name != "kqn" and model_name != "lpkt":
             df["hidden"] = [np.array(a) for a in hs[0][bz].cpu().tolist()]
         elif model_name == "kqn":
             df["ek"] = [np.array(a) for a in hs[0][bz].cpu().tolist()]
             df["es"] = [np.array(a) for a in hs[1][bz].cpu().tolist()]
+        elif model_name == "lpkt":
+            df["h"] = [np.array(a) for a in hs[0][bz].cpu().tolist()]
+            df["e_data"] = [np.array(a) for a in hs[1][bz].cpu().tolist()]
 
         df = df[df["select"] != 0]
         alldfs.append(df)
@@ -228,10 +267,10 @@ def group_fusion(dmerge, model, model_name, fusion_type, fout):
     for key in dmerge.keys():
         if key == "hs":
             drest[key] = []
-            if model_name in ["dkvmn", "akt", "saint", "sakt"]:
+            if model_name in hasearly and model_name != "kqn" and model_name != "lpkt":
                 drest[key] = [dmerge[key][0][rest_start:]]
-            elif model_name == "kqn":
-                drest[key] = [dmerge[key][0][rest_start:], dmerge[key][1][rest_start:]]                
+            elif model_name == "kqn" or model_name == "lpkt":
+                drest[key] = [dmerge[key][0][rest_start:], dmerge[key][1][rest_start:]]              
         else:
             drest[key] = dmerge[key][rest_start:] 
     restlen = drest["cr"].shape[0]
@@ -243,7 +282,7 @@ def group_fusion(dmerge, model, model_name, fusion_type, fout):
                 dfs.setdefault(key, [])
                 dfs[key].extend([row[key]])
     df = pd.DataFrame(dfs)
-    print(f"real bz: {realbz}, effective_dfs: {len(effective_dfs)}, rest_start: {rest_start}, drestlen: {restlen}, predict infos: {df.shape}")
+    # print(f"real bz: {realbz}, effective_dfs: {len(effective_dfs)}, rest_start: {rest_start}, drestlen: {restlen}, predict infos: {df.shape}")
 
     if df.shape[0] == 0:
         return {}, drest
@@ -254,7 +293,7 @@ def group_fusion(dmerge, model, model_name, fusion_type, fout):
     for key in dres:
         dfinal[key] = np.concatenate(dres[key], axis=0)
     early = False
-    if model_name in ["dkvmn", "akt", "saint", "sakt", "kqn"]:
+    if model_name in hasearly and "early_fusion" in fusion_type:
         early = True
     save_question_res(dfinal, fout, early)
     return dfinal , drest
@@ -278,9 +317,10 @@ def evaluate_question(model, test_loader, model_name, fusion_type=["early_fusion
     # dkvmn / akt / saint: give cur -> predict cur
     # sakt: give past+cur -> predict cur
     # kqn: give past+cur -> predict cur
+    hasearly = ["dkvmn", "skvmn", "kqn", "akt", "saint", "sakt", "hawkes", "akt_vector", "akt_norasch", "akt_mono", "akt_attn", "aktattn_pos", "aktmono_pos", "akt_raschx", "akt_raschy", "aktvec_raschx", "lpkt"]
     if save_path != "":
         fout = open(save_path, "w", encoding="utf8")
-        if model_name in ["dkvmn", "akt", "saint", "sakt", "kqn"]:
+        if model_name in hasearly:
             fout.write("\t".join(["orirow", "qidx", "questions", "concepts", "concept_preds", "late_trues", "late_mean", "late_vote", "late_all", "early_trues", "early_preds"]) + "\n")
         else:
             fout.write("\t".join(["orirow", "qidx", "questions", "concepts", "concept_preds", "late_trues", "late_mean", "late_vote", "late_all"]) + "\n")
@@ -294,14 +334,18 @@ def evaluate_question(model, test_loader, model_name, fusion_type=["early_fusion
         lenc = 0
         for data in test_loader:
             if model_name in ["dkt_forget"]:
-                q, c, r, qshft, cshft, rshft, m, sm, d, dshft, dqtest = data
+                dcurori, dgaps, dqtest = data
             else:
-                q, c, r, qshft, cshft, rshft, m, sm, dqtest = data
+                dcurori, dqtest = data
+
+            q, c, r = dcurori["qseqs"], dcurori["cseqs"], dcurori["rseqs"]
+            qshft, cshft, rshft = dcurori["shft_qseqs"], dcurori["shft_cseqs"], dcurori["shft_rseqs"]
+            m, sm = dcurori["masks"], dcurori["smasks"]
             q, c, r, qshft, cshft, rshft, m, sm = q.to(device), c.to(device), r.to(device), qshft.to(device), cshft.to(device), rshft.to(device), m.to(device), sm.to(device)
             qidxs, rests, orirow = dqtest["qidxs"], dqtest["rests"], dqtest["orirow"]
             lenc += q.shape[0]
-            print("="*20)
-            print(f"start predict seqlen: {lenc}")
+            # print("="*20)
+            # print(f"start predict seqlen: {lenc}")
             model.eval()
 
             # print(f"before y: {y.shape}")
@@ -309,10 +353,10 @@ def evaluate_question(model, test_loader, model_name, fusion_type=["early_fusion
             cc = torch.cat((c[:,0:1], cshft), dim=1)
             cr = torch.cat((r[:,0:1], rshft), dim=1)
             dcur = dict()
-            if model_name in ["dkvmn"]:
+            if model_name in ["dkvmn", "skvmn"]:
                 y, h = model(cc.long(), cr.long(), True)
                 y = y[:,1:]
-            elif model_name == "akt":
+            elif model_name in ["akt", "akt_vector", "akt_norasch", "akt_mono", "akt_attn", "aktattn_pos", "aktmono_pos", "akt_raschx", "akt_raschy", "aktvec_raschx"]:
                 y, reg_loss, h = model(cc.long(), cr.long(), cq.long(), True)
                 y = y[:,1:]
             elif model_name == "saint":
@@ -333,14 +377,24 @@ def evaluate_question(model, test_loader, model_name, fusion_type=["early_fusion
                 y = model(c.long(), r.long())
                 y = (y * one_hot(cshft.long(), model.num_c)).sum(-1)
             elif model_name in ["dkt_forget"]:
-                y = model(c.long(), r.long(), d, dshft)
+                y = model(c.long(), r.long(), dgaps)
                 y = (y * one_hot(cshft.long(), model.num_c)).sum(-1)
             elif model_name in ["atkt", "atktfix"]:
                 y, _ = model(c.long(), r.long())
                 y = (y * one_hot(cshft.long(), model.num_c)).sum(-1)
             elif model_name == "gkt":
                 y = model(cc.long(), cr.long())
-            
+            elif model_name == "hawkes":
+                ct = torch.cat((dcurori["tseqs"][:,0:1], dcurori["shft_tseqs"]), dim=1)
+                y = model(cc.long(), cq.long(), ct.long(), cr.long(), True)
+                y, h = y[:, 1:]
+            elif model_name == "lpkt":
+                cit = torch.cat((dcur["itseqs"][:,0:1], dcur["shft_itseqs"]), dim=1)
+                y, h, e_data = model(cq.long(), cr.long(), cit.long(), True)
+                start_hemb = torch.tensor([-1] * (h.shape[0] * h.shape[2])).reshape(h.shape[0], 1, h.shape[2]).to(device) # add the first hidden emb
+                h = torch.cat((start_hemb, h), dim=1)
+                # e_data = torch.cat((start_hemb, e_data), dim=1)
+                y = y[:, 1:]
 
             concepty = torch.masked_select(y, sm).detach().cpu()
             conceptt = torch.masked_select(rshft, sm).detach().cpu()
@@ -352,7 +406,9 @@ def evaluate_question(model, test_loader, model_name, fusion_type=["early_fusion
             hs = []
             if model_name == "kqn":
                 hs = [ek, es]
-            elif model_name in ["dkvmn", "akt", "saint", "sakt"]:
+            elif model_name == "lpkt":
+                hs = [h, e_data]
+            elif model_name in hasearly:
                 hs = [h]
             dcur["hs"], dcur["sm"], dcur["cq"], dcur["cc"], dcur["cr"], dcur["y"], dcur["qidxs"], dcur["rests"], dcur["orirow"] = hs, sm, cq, cc, cr, y, qidxs, rests, orirow
             # merge history
@@ -367,7 +423,11 @@ def evaluate_question(model, test_loader, model_name, fusion_type=["early_fusion
                             dmerge[key] = [[], []]
                             dmerge[key][0] = torch.cat((dhistory[key][0], dcur[key][0]), dim=0)
                             dmerge[key][1] = torch.cat((dhistory[key][1], dcur[key][1]), dim=0)
-                        elif model_name in ["dkvmn", "akt", "saint", "sakt"]:
+                        elif model_name == "lpkt":
+                            dmerge[key] = [[], []]
+                            dmerge[key][0] = torch.cat((dhistory[key][0], dcur[key][0]), dim=0)
+                            dmerge[key][1] = torch.cat((dhistory[key][1], dcur[key][1]), dim=0)
+                        elif model_name in hasearly:
                             dmerge[key] = [torch.cat((dhistory[key][0], dcur[key][0]), dim=0)]                            
                     else:
                         dmerge[key] = torch.cat((dhistory[key], dcur[key]), dim=0)
@@ -385,20 +445,20 @@ def evaluate_question(model, test_loader, model_name, fusion_type=["early_fusion
         aucs, accs = dict(), dict()
         ts = np.concatenate(y_trues, axis=0)
         ps = np.concatenate(y_scores, axis=0)
-        print(f"ts.shape: {ts.shape}, ps.shape: {ps.shape}")
+        # print(f"ts.shape: {ts.shape}, ps.shape: {ps.shape}")
         auc = metrics.roc_auc_score(y_true=ts, y_score=ps)
         prelabels = [1 if p >= 0.5 else 0 for p in ps]
         acc = metrics.accuracy_score(ts, prelabels)
         aucs["concepts"] = auc
         accs["concepts"] = acc
 
-        print(f"dinfos: {dinfos.keys()}")
+        # print(f"dinfos: {dinfos.keys()}")
         for key in dinfos:
             if key not in ["late_mean", "late_vote", "late_all", "early_preds"]:
                 continue
             ts = np.concatenate(dinfos['late_trues'], axis=0) # early_trues == late_trues
             ps = np.concatenate(dinfos[key], axis=0)
-            print(f"key: {key}, ts.shape: {ts.shape}, ps.shape: {ps.shape}")
+            # print(f"key: {key}, ts.shape: {ts.shape}, ps.shape: {ps.shape}")
             auc = metrics.roc_auc_score(y_true=ts, y_score=ps)
             prelabels = [1 if p >= 0.5 else 0 for p in ps]
             acc = metrics.accuracy_score(ts, prelabels)
@@ -456,6 +516,8 @@ def get_info_dkt_forget(row, data_config):
 def evaluate_splitpred_question(model, data_config, testf, model_name, save_path="", use_pred=False, train_ratio=0.2, atkt_pad=False):
     if save_path != "":
         fout = open(save_path, "w", encoding="utf8")
+    if model_name == "lpkt":
+        at2idx, it2idx = generate_time2idx(data_config)
     with torch.no_grad():
         y_trues = []
         y_scores = []
@@ -479,6 +541,11 @@ def evaluate_splitpred_question(model, data_config, testf, model_name, save_path
             is_repeat = ["0"] * curl if "is_repeat" not in row else row["is_repeat"].split(",")
             is_repeat = [int(s) for s in is_repeat]
             questions = [] if "questions" not in row else row["questions"].split(",")
+            times = [] if "timestamps" not in row else row["timestamps"].split(",")
+            if model_name == "lpkt":
+                shft_times = [0] + times[:-1]
+                it_times = np.maximum(np.minimum((np.array(timestamps) - np.array(shft_timestamps)) // 60, 43200),-1)
+                it_times = [it2idx[str(t)] for t in it]
 
             qlen, qtrainlen, ctrainlen = get_cur_teststart(is_repeat, train_ratio)
             # print(f"idx: {idx}, qlen: {qlen}, qtrainlen: {qtrainlen}, ctrainlen: {ctrainlen}")
@@ -487,9 +554,20 @@ def evaluate_splitpred_question(model, data_config, testf, model_name, save_path
             cq = torch.tensor([int(s) for s in questions]).to(device)
             cc = torch.tensor([int(s) for s in concepts]).to(device)
             cr = torch.tensor([int(s) for s in responses]).to(device)
+            ct = torch.tensor([int(s) for s in times]).to(device)
+            dtotal = {"cq": cq, "cc": cc, "cr": cr, "ct": ct}
+            if model_name == "lpkt":
+                cit = torch.tensor([int(s) for s in it_times]).to(device)
+                dtotal["cit"] = cit
             # print(f"cc: {cc[0:ctrainlen]}")
             curcin, currin = cc[0:ctrainlen].unsqueeze(0), cr[0:ctrainlen].unsqueeze(0)
             curqin = cq[0:ctrainlen].unsqueeze(0) if cq.shape[0] > 0 else cq
+            curtin = ct[0:ctrainlen].unsqueeze(0) if ct.shape[0] > 0 else ct
+            if model_name == "lpkt":
+                curitin = ct[0:ctrainlen].unsqueeze(0) if cit.shape[0] > 0 else cit
+            dcur = {"curqin": curqin, "curcin": curcin, "currin": currin, "curtin": curtin}
+            if model_name == "lpkt":
+                dcur["curitin"] = curitin
             curdforget = dict()
             for key in dforget:
                 dforget[key] = torch.tensor(dforget[key]).to(device)
@@ -502,7 +580,9 @@ def evaluate_splitpred_question(model, data_config, testf, model_name, save_path
             if not use_pred:
                 uid, end = row["uid"], curl
                 qidx = qtrainlen
-                qidxs, ctrues, cpreds = predict_each_group2(curdforget, dforget, is_repeat, qidx, uid, idx, curqin, curcin, currin, model_name, model, t, cq, cc, cr, end, fout, atkt_pad)
+                # qidxs, ctrues, cpreds = predict_each_group2(curdforget, dforget, is_repeat, qidx, uid, idx, curqin, curcin, currin, model_name, model, t, cq, cc, cr, end, fout, atkt_pad)
+                # qidxs, ctrues, cpreds = predict_each_group2(curdforget, dforget, is_repeat, qidx, uid, idx, dcur, model_name, model, t, dtotal, end, fout, atkt_pad)
+                qidxs, ctrues, cpreds = predict_each_group2(dtotal, dcur, dforget, curdforget, is_repeat, qidx, uid, idx, model_name, model, t, end, fout, atkt_pad)
                 # 计算
                 save_currow_question_res(idx, dcres, dqres, qidxs, ctrues, cpreds, uid, fout)
             else:
@@ -520,13 +600,8 @@ def evaluate_splitpred_question(model, data_config, testf, model_name, save_path
                     end = rtmp[-1]+1
                     uid = row["uid"]
                     # if use_pred:
-                    curqin, curcin, currin, curdforget, ctrues, cpreds = predict_each_group(curdforget, dforget, is_repeat, qidx, uid, idx, curqin, curcin, currin, model_name, model, t, cq, cc, cr, end, fout, atkt_pad)
-                    # else:
-                    #     ctrues, cpreds = [], []
-                    #     for m in range(t, end):
-                    #         nextqin, nextcin, nextrin, nextdforget, ctrue, cpred = predict_each_group(curdforget, dforget, is_repeat, qidx, uid, idx, curqin, curcin, currin, model_name, model, m, cq, cc, cr, m+1, fout)
-                    #         ctrues.extend(ctrue)
-                    #         cpreds.extend(cpred)
+                    # curqin, curcin, currin, curdforget, ctrues, cpreds = predict_each_group(curdforget, dforget, is_repeat, qidx, uid, idx, curqin, curcin, currin, model_name, model, t, cq, cc, cr, end, fout, atkt_pad)
+                    curqin, curcin, currin, curdforget, ctrues, cpreds = predict_each_group(dtotal, dcur, dforget, curdforget, is_repeat, qidx, uid, idx, model_name, model, t, end, fout, atkt_pad)
                     
                     late_mean, late_vote, late_all = save_each_question_res(dcres, dqres, ctrues, cpreds)    
                     # print("\t".join([str(idx), str(uid), str(k), str(qidx), str(late_mean), str(late_vote), str(late_all)]))    
@@ -564,15 +639,24 @@ def get_cur_teststart(is_repeat, train_ratio):
             break
     return qlen, qtrainlen, ctrainlen
 
-def predict_each_group(curdforget, dforget, is_repeat, qidx, uid, idx, curqin, curcin, currin, model_name, model, t, cq, cc, cr, end, fout, atkt_pad=False, maxlen=200):
+# def predict_each_group(curdforget, dforget, is_repeat, qidx, uid, idx, curqin, curcin, currin, model_name, model, t, cq, cc, cr, end, fout, atkt_pad=False, maxlen=200):
+def predict_each_group(dtotal, dcur, dforget, curdforget, is_repeat, qidx, uid, idx, model_name, model, t, end, fout, atkt_pad=False, maxlen=200):
     """use the predict result as next question input
     """
+    curqin, curcin, currin, curtin = dcur["curqin"], dcur["curcin"], dcur["currin"], dcur["curtin"]
+    cq, cc, cr, ct = dtotal["cq"], dtotal["cc"], dtotal["cr"], dtotal["ct"]
+    if model_name == "lpkt":
+        curitin = dcur["curitin"]
+        cit = dtotal["cit"]
+
     nextcin, nextrin = curcin, currin
     import copy
     nextdforget = copy.deepcopy(curdforget)
     ctrues, cpreds = [], []
     for k in range(t, end):
-        qin, cin, rin = curqin, curcin, currin
+        qin, cin, rin, tin = curqin, curcin, currin, curtin
+        if model_name == "lpkt":
+            itin = curitin
         # 输入长度大于200时，截断
         # print("cin: ", cin)
         start = 0
@@ -584,9 +668,16 @@ def predict_each_group(curdforget, dforget, is_repeat, qidx, uid, idx, curqin, c
         # print(f"start: {start}, cin: {cin.shape}")
         if cq.shape[0] > 0:
             qin = qin[:, start:]
+        if ct.shape[0] > 0:
+            tin = tin[:, start:]
+        if model_name == "lpkt":
+            itin = itin[:, start:]
         # print(f"start: {start}, cin: {cin.shape}")
         cout, true = cc.long()[k], cr.long()[k] # 当前预测的是第k个
         qout = None if cq.shape[0] == 0 else cq.long()[k]
+        tout = None if ct.shape[0] == 0 else ct.long()[k]
+        if model_name == "lpkt":
+            itout = None if cit.shape[0] == 0 else cit.long()[k]
         if model_name in ["dkt", "dkt+"]:
             y = model(cin.long(), rin.long())
             # print(y)
@@ -638,14 +729,14 @@ def predict_each_group(curdforget, dforget, is_repeat, qidx, uid, idx, curqin, c
                 pred = y[0][oricinlen-1][cout.item()]
             else:
                 pred = y[0][-1][cout.item()]
-        elif model_name in ["dkvmn"]:
+        elif model_name in ["dkvmn", "skvmn"]:
             curc, curr = torch.tensor([[cout.item()]]).to(device), torch.tensor([[true.item()]]).to(device)
             cin, rin = torch.cat((cin, curc), axis=1), torch.cat((rin, curr), axis=1)
             # print(f"cin: {cin.shape}, curc: {curc.shape}")
             # 应该用预测的r更新memory value，但是这里一个知识点一个知识点预测，所以curr不起作用！
             y = model(cin.long(), rin.long())
             pred = y[0][-1]
-        elif model_name == "akt":  
+        elif model_name in ["akt", "akt_vector", "akt_norasch", "akt_mono", "akt_attn", "aktattn_pos", "aktmono_pos", "akt_raschx", "akt_raschy", "aktvec_raschx"]:  
             #### 输入有question！     
             if qout != None:
                 curq = torch.tensor([[qout.item()]]).to(device)
@@ -656,12 +747,32 @@ def predict_each_group(curdforget, dforget, is_repeat, qidx, uid, idx, curqin, c
 
             y, reg_loss = model(cin.long(), rin.long(), qin.long())
             pred = y[0][-1]
+        elif model_name == "lpkt":
+            if itout != None:
+                curit = torch.tensor([[itout.item()]]).to(device)
+                itin = torch.cat((itin, curit), axis=1)
+            curc, curr = torch.tensor([[cout.item()]]).to(device), torch.tensor([[1]]).to(device)
+            cin, rin = torch.cat((cin, curc), axis=1), torch.cat((rin, curr), axis=1)
+            y = model(cin.long(), rin.long(), itin.long())
+            pred = y[0][-1]
         elif model_name == "gkt":
             curc, curr = torch.tensor([[cout.item()]]).to(device), torch.tensor([[1]]).to(device)
             cin, rin = torch.cat((cin, curc), axis=1), torch.cat((rin, curr), axis=1)
             y = model(cin.long(), rin.long())
             # print(f"y.shape is {y.shape},cin shape is {cin.shape}")
             pred = y[0][-1]
+        elif model_name == "hawkes":
+            curc, curr = torch.tensor([[cout.item()]]).to(device), torch.tensor([[1]]).to(device)
+            if tout != None:
+                curt = torch.tensor([[tout.item()]]).to(device)
+                tin = torch.cat((tin, curt), axis=1)
+            if qout != None:
+                curq = torch.tensor([[qout.item()]]).to(device)
+                qin = torch.cat((qin, curq), axis=1)
+            curc, curr = torch.tensor([[cout.item()]]).to(device), torch.tensor([[1]]).to(device)
+            cin, rin = torch.cat((cin, curc), axis=1), torch.cat((rin, curr), axis=1)
+            y = model(cin.long(), qin.long(), tin.long(), rin.long())
+            pred = y[0][-1]
         
         predl = 1 if pred.item() >= 0.5 else 0
         cpred = torch.tensor([[predl]]).to(device)
@@ -734,9 +845,15 @@ def cal_predres(dcres, dqres):
         dres[key] = [len(preds), auc, acc]
     return dres
 
-def prepare_data(model_name, is_repeat, qidx, curqin, curcin, currin, curdforget, cq, cc, cr, dforget, t, end, maxlen=200):
-    dqshfts, dcshfts, drshfts, dds, ddshfts = [], [], [], dict(), dict()
-    dqs, dcs, drs = [], [], []
+def prepare_data(model_name, is_repeat, qidx, dcur, curdforget, dtotal, dforget, t, end, maxlen=200):
+    curqin, curcin, currin, curtin = dcur["curqin"], dcur["curcin"], dcur["currin"], dcur["curtin"]
+    cq, cc, cr, ct = dtotal["cq"], dtotal["cc"], dtotal["cr"], dtotal["ct"]
+    dqshfts, dcshfts, drshfts, dtshfts, dds, ddshfts = [], [], [], [], dict(), dict()
+    dqs, dcs, drs, dts = [], [], [], []
+    if model_name == "lpkt":
+        curitin = dcur["curitin"]
+        cit = dtotal["cit"]
+        dits, ditshfts = [], []
     qidxs = []
     qstart = qidx-1
     for k in range(t, end):
@@ -766,6 +883,21 @@ def prepare_data(model_name, is_repeat, qidx, curqin, curcin, currin, curdforget
             dqs.append(curqin[:, start:])
             curq = torch.cat((curqin[:, start+1:], curq), axis=1)
             dqshfts.append(curq)
+        if ct.shape[0] > 0:
+            curt = ct.long()[k]
+            curt = torch.tensor([[curt.item()]]).to(device)
+
+            dts.append(curtin[:, start:])
+            curt = torch.cat((curtin[:, start+1:], curt), axis=1)
+            dtshfts.append(curt)
+        if model_name == "lpkt":
+            if cit.shape[0] > 0:
+                curit = cit.long()[k]
+                curit = torch.tensor([[curit.item()]]).to(device)
+
+                dits.append(curitin[:, start:])
+                curit = torch.cat((curitin[:, start+1:], curit), axis=1)
+                ditshfts.append(curit)
 
         d, dshft = dict(), dict()
         if model_name == "dkt_forget":
@@ -781,9 +913,13 @@ def prepare_data(model_name, is_repeat, qidx, curqin, curcin, currin, curdforget
         
     finalcs, finalrs = torch.cat(dcs, axis=0), torch.cat(drs, axis=0)
     finalqs, finalqshfts = torch.tensor([]), torch.tensor([])
+    finalts, finaltshfts = torch.tensor([]), torch.tensor([]) 
     if cq.shape[0] > 0:
         finalqs = torch.cat(dqs, axis=0)
         finalqshfts = torch.cat(dqshfts, axis=0)
+    if ct.shape[0] > 0:
+        finalts = torch.cat(dts, axis=0)
+        finaltshfts = torch.cat(dtshfts, axis=0)
     finalcshfts, finalrshfts = torch.cat(dcshfts, axis=0), torch.cat(drshfts, axis=0)
     finald, finaldshft = dict(), dict()
     for key in dds:
@@ -791,18 +927,34 @@ def prepare_data(model_name, is_repeat, qidx, curqin, curcin, currin, curdforget
         finaldshft[key] = torch.cat(ddshfts[key], axis=0)
     # print(f"qidx: {len(qidxs)}, finalqs: {finalqs.shape}, finalcs: {finalcs.shape}, finalrs: {finalrs.shape}")
     # print(f"qidx: {len(qidxs)}, finalqshfts: {finalqshfts.shape}, finalcshfts: {finalcshfts.shape}, finalrshfts: {finalrshfts.shape}")
-    return qidxs, finalqs, finalcs, finalrs, finalqshfts, finalcshfts, finalrshfts, finald, finaldshft     
+    if model_name == "lpkt":
+        finalits, finalitshfts = torch.tensor([]), torch.tensor([])
+        if cit.shape[0] > 0:
+            finalits = torch.cat(dits, axis=0)
+            finalitshfts = torch.cat(ditshfts, axis=0)
+    if model_names != lpkt:
+        return qidxs, finalqs, finalcs, finalrs, finalts, finalqshfts, finalcshfts, finalrshfts, finaltshfts, finald, finaldshft  
+    else: 
+        return qidxs, finalqs, finalcs, finalrs, finalts, finalits, finalqshfts, finalcshfts, finalrshfts, finaltshfts, finalitshfts, finald, finaldshft
 
-def predict_each_group2(curdforget, dforget, is_repeat, qidx, uid, idx, curqin, curcin, currin, model_name, model, t, cq, cc, cr, end, fout, atkt_pad=False, maxlen=200):
+# def predict_each_group2(curdforget, dforget, is_repeat, qidx, uid, idx, curqin, curcin, currin, model_name, model, t, cq, cc, cr, end, fout, atkt_pad=False, maxlen=200):
+def predict_each_group2(dtotal, dcur, dforget, curdforget, is_repeat, qidx, uid, idx, model_name, model, t, end, fout, atkt_pad=False, maxlen=200):
     """not use the predict result
     """
+    curqin, curcin, currin, curtin = dcur["curqin"], dcur["curcin"], dcur["currin"], dcur["curtin"]
+    cq, cc, cr, ct = dtotal["cq"], dtotal["cc"], dtotal["cr"], dtotal["ct"]
+    if model_name == "lpkt":
+        cit = dtotal["cit"]
     nextcin, nextrin = curcin, currin
     import copy
     nextdforget = copy.deepcopy(curdforget)
     ctrues, cpreds = [], []
     # 以下这些用的是同一个历史,可以并行
     # 不用预测结果
-    qidxs, finalqs, finalcs, finalrs, finalqshfts, finalcshfts, finalrshfts, finald, finaldshft = prepare_data(model_name, is_repeat, qidx, curqin, curcin, currin, curdforget, cq, cc, cr, dforget, t, end)
+    if model_name == "lpkt":
+        qidxs, finalqs, finalcs, finalrs, finalts, finalits, finalqshfts, finalcshfts, finalrshfts, finaltshfts, finalitshfts, finald, finaldshft = prepare_data(model_name, is_repeat, qidx, dcur, curdforget, dtotal, dforget, t, end)
+    else:
+        qidxs, finalqs, finalcs, finalrs, finalts, finalqshfts, finalcshfts, finalrshfts, finaltshfts, finald, finaldshft = prepare_data(model_name, is_repeat, qidx, dcur, curdforget, dtotal, dforget, t, end)
     bidx, bz = 0, 128
     while bidx < finalcs.shape[0]:
         curc, curr = finalcs[bidx: bidx+bz], finalrs[bidx: bidx+bz]
@@ -812,22 +964,29 @@ def predict_each_group2(curdforget, dforget, is_repeat, qidx, uid, idx, curqin,
         if finalqs.shape[0] > 0:
             curq = finalqs[bidx: bidx+bz]
             curqshft = finalqshfts[bidx: bidx+bz]
+        if finalts.shape[0] > 0:
+            curt = finalts[bidx: bidx+bz]
+            curtshft = finaltshfts[bidx: bidx+bz]
         curd, curdshft = dict(), dict()
         if model_name == "dkt_forget":
             for key in finald:
                 curd[key] = finald[key][bidx: bidx+bz]
                 curdshft[key] = finaldshft[key][bidx: bidx+bz]
+        if model_name == "lpkt":
+            curit = finalits[bidx: bidx+bz]
+            curitshft = finalitshfts[bidx: bidx+bz]
         ## start predict
         ccq = torch.cat((curq[:,0:1], curqshft), dim=1)
         ccc = torch.cat((curc[:,0:1], curcshft), dim=1)
         ccr = torch.cat((curr[:,0:1], currshft), dim=1)
+        cct = torch.cat((curt[:,0:1], curtshft), dim=1)
         if model_name in ["dkt", "dkt+"]:
             y = model(curc.long(), curr.long())
             y = (y * one_hot(curcshft.long(), model.num_c)).sum(-1)
         elif model_name in ["dkt_forget"]:
             y = model(curc.long(), curr.long(), curd, curdshft)
             y = (y * one_hot(curcshft.long(), model.num_c)).sum(-1)
-        elif model_name in ["dkvmn"]:
+        elif model_name in ["dkvmn", "skvmn"]:
             y = model(ccc.long(), ccr.long())
             y = y[:,1:]
         elif model_name in ["kqn", "sakt"]:
@@ -835,7 +994,7 @@ def predict_each_group2(curdforget, dforget, is_repeat, qidx, uid, idx, curqin,
         elif model_name == "saint":
             y = model(ccq.long(), ccc.long(), curr.long())
             y = y[:, 1:]
-        elif model_name == "akt":                                
+        elif model_name in ["akt", "akt_vector", "akt_norasch", "akt_mono", "akt_attn", "aktattn_pos", "aktmono_pos", "akt_raschx", "akt_raschy", "aktvec_raschx"]:                                
             y, reg_loss = model(ccc.long(), ccr.long(), ccq.long())
             y = y[:,1:]
         elif model_name in ["atkt", "atktfix"]:
@@ -850,10 +1009,17 @@ def predict_each_group2(curdforget, dforget, is_repeat, qidx, uid, idx, curqin,
                 curcshft = torch.cat((curcshft, pad), axis=1)
             y, _ = model(curc.long(), curr.long())
             y = (y * one_hot(curcshft.long(), model.num_c)).sum(-1)
+        elif model_name == "lpkt":
+            ccit = torch.cat((curit[:,0:1], curitshft), dim=1)
+            y = model(ccq.long(), ccr.long(), ccit.long())
+            y = y[:, 1:]
         elif model_name == "gkt":
             y = model(ccc.long(), ccr.long())
             # print(f"y: {y}")
             # y = y[:, t-1:t]
+        elif model_name == "hawkes":
+            y = model(ccc.long(), ccq.long(), cct.long(), ccr.long())
+            pred = y[0][-1]
         if model_name in ["atkt", "atktfix"] and atkt_pad == True:
             # print(f"use idx: {oricurclen-1}")
             pred = y[:, oricurclen-1].tolist()
@@ -899,4 +1065,4 @@ def save_currow_question_res(idx, dcres, dqres, qidxs, ctrues, cpreds, uid, fout
         ctrues, cpreds = dqidx[qidx]["trues"], dqidx[qidx]["preds"]
         late_mean, late_vote, late_all = save_each_question_res(dcres, dqres, ctrues, cpreds)
         # print("\t".join([str(idx), str(uid), str(qidx), str(late_mean), str(late_vote), str(late_all)]))
-        fout.write("\t".join([str(idx), str(uid), str(qidx), str(late_mean), str(late_vote), str(late_all)]) + "\n")
+        fout.write("\t".join([str(idx), str(uid), str(qidx), str(late_mean), str(late_vote), str(late_all)]) + "\n")
\ No newline at end of file
