@@ -1,5 +1,4 @@
 import os, sys
-from .split_datasets import main as split
 
 def process_raw_data(dataset_name,dname2paths):
     readf = dname2paths[dataset_name]
@@ -8,6 +7,8 @@ def process_raw_data(dataset_name,dname2paths):
     print(f"Start preprocessing data: {dataset_name}")
     if dataset_name == "assist2009":
         from .assist2009_preprocess import read_data_from_csv
+    elif dataset_name == "assist2012":
+        from .assist2012_preprocess import read_data_from_csv
     elif dataset_name == "assist2015":
         from .assist2015_preprocess import read_data_from_csv
     elif dataset_name == "algebra2005":
@@ -20,11 +21,20 @@ def process_raw_data(dataset_name,dname2paths):
         from .nips_task34_preprocess import read_data_from_csv
     elif dataset_name == "poj":
         from .poj_preprocess import read_data_from_csv
-
-    if dataset_name != "nips_task34":
+    elif dataset_name == "slepemapy":
+        from .slepemapy_preprocess import read_data_from_csv
+    elif dataset_name == "assist2017":
+        from .assist2017_preprocess import read_data_from_csv
+    elif dataset_name == "junyi2015":
+        from .junyi2015_preprocess import read_data_from_csv, load_q2c
+    
+    if dataset_name == "junyi2015":
+        dq2c = load_q2c(readf.replace("junyi_ProblemLog_original.csv","junyi_Exercise_table.csv"))
+        read_data_from_csv(readf, writef, dq2c)
+    elif dataset_name != "nips_task34":
         read_data_from_csv(readf, writef)
     else:
         metap = os.path.join(dname, "metadata")
         read_data_from_csv(readf, metap, "task_3_4", writef)
      
-    return dname,writef
\ No newline at end of file
+    return dname,writef
